//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Main.h"
#include "math.h"
#include "stdio.h"
#include "AutoGRAPH_OCX.h"
#include "CheckPointProperties.h"
#include "PointsList.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TAutoGRAPHAutomation *Automation = NULL;
TMainForm *MainForm;
const char SENS_ON[] = "", SENS_OFF[] = "";
//---------------------------------------------------------------------------
AnsiString __fastcall GetCoordStr(double Value)
{
AnsiString Str;
int IValue = Value; Value -= IValue;
div_t Min = div(Value * 6000000, 100000);
Min.rem += 50, Min.rem /= 100;
Str.printf("%u %02u,%03u", IValue, Min.quot, Min.rem);
return Str;
}
//---------------------------------------------------------------------------
AnsiString __fastcall ContinuanceStrMin(TDateTime Continuance)
{
AnsiString Str;
if ((double)Continuance >= 1 / 1440.0 && SecondOf(Continuance) >= 30)
            Continuance += 1 / 1440.0;
WORD Hours, Mins, Secs, MSecs;
DecodeTime(Continuance, Hours, Mins, Secs, MSecs);
if      (Hours) Str.printf("%u:%02u ", Hours, Mins);
else if (Mins)  Str.printf(     "%u ", Mins);
else            Str.printf(       "%u ", Secs);
return Str;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner) : TForm(Owner)
{
PrevCarDevice = 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
DestroyAutomation();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateAutomation()
{
if (!Automation)
        {
        Automation = new TAutoGRAPHAutomation(Owner);
        Automation->OnChangeCarsLists   = AutomationChangeCarsLists;
        Automation->OnChangeParameters  = AutomationChangeParameters;
        Automation->OnEndComputing      = AutomationEndComputing;
        Automation->OnReceiveOnlineData = AutomationReceiveOnlineData;
        Automation->OnCloseServer       = AutomationCloseServer;
        Automation->Connect();
        ComputingTimerTimer(ComputingTimer);
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyAutomation()
{
if (Automation)
        {
        Automation->Disconnect();
        delete Automation;
        Automation = NULL;
        ComputingTimerTimer(ComputingTimer);
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ConnectButtonClick(TObject *Sender)
{
CreateAutomation();
ShowApplicationButton->Enabled = true;
HideApplicationButton->Enabled = true;
VersionPanel->Caption = Automation->CurrentVersion;
FillGroupsList();

LastPositionButton->Enabled = true;
LastTrackButton->Enabled    = true;
SetChPPrmsButton->Enabled   = true;
ComputingButton->Enabled    = true;

LoadChPFromFileButton->Enabled  = true;
SaveChPToFileButton->Enabled    = true;
ClearCheckPointsButton->Enabled = true;
AddCheckPointButton->Enabled    = true;

FillTripsInfoList();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DisconnectButtonClick(TObject *Sender)
{
DestroyAutomation();
ShowApplicationButton->Enabled = false;
HideApplicationButton->Enabled = false;
VersionPanel->Caption = "- - -";
GroupsList->Items->Clear();
GroupsComboBox->Items->Clear();
CarsComboBox->Items->Clear();
ShowPanels();

LastPositionButton->Enabled = false;
LastTrackButton->Enabled    = false;
SetChPPrmsButton->Enabled   = false;
ComputingButton->Enabled    = false;

ComputingResultInfoLabel->Caption = "";
ComputingResultButton->Enabled = false;

ChPListView->Items->Clear();
LoadChPFromFileButton->Enabled  = false;
SaveChPToFileButton->Enabled    = false;
ClearCheckPointsButton->Enabled = false;
AddCheckPointButton->Enabled    = false;
ChangeCheckPointButton->Enabled = false;
DeleteCheckPointButton->Enabled = false;

TripsListView->Items->Clear();
EntriesTypeBox->Enabled     = false;
EntriesButton->Enabled      = false;
ExportToFileButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AutomationChangeCarsLists(TObject *Sender)
{
FillGroupsList();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AutomationChangeParameters(TObject *Sender)
{             
TTreeNode *Node = GroupsList->Selected;
if (Node) if (Node->Level == 1)
        FillInfoListView(Node);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AutomationEndComputing(TObject *Sender)
{           
ComputingTimerTimer(ComputingTimer);
if (Automation)
        {
        if (Automation->ComputingCheck)
                {
                ComputingResultInfoLabel->Caption = "OK";
                ComputingResultButton->Enabled = true;
                }
        else    {
                ComputingResultInfoLabel->Caption = "";
                ComputingResultButton->Enabled = false;
                }
        FillTripsInfoList();
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AutomationReceiveOnlineData(TObject *Sender)
{                      
TTreeNode *Node = GroupsList->Selected;
if (Node) if (Node->Level == 1)
        {
        FillInfoListView(Node);
        FillOnlineInfoListView(Node);
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AutomationCloseServer(TObject *Sender)
{
DisconnectButton->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FillGroupsList()
{
GroupsList->OnChange = NULL;
TTreeNodes *Items = GroupsList->Items;
Items->BeginUpdate();
Items->Clear();

if (Automation)
{
TTreeNode *SelGroupItem = NULL, *SelCarItem = NULL;
for (DWORD GroupsNum = Automation->GroupsNum,
     GroupIndex = 1; GroupIndex <= GroupsNum; GroupIndex ++)
        {
        Automation->GroupIndex = GroupIndex;
        TTreeNode *Item = Items->AddObject(NULL, Automation->GroupName, (void *)GroupIndex);
        bool ThisGroup = !AnsiCompareFileName(Automation->GroupFileName, PrevGroupFileName);
        if (ThisGroup) SelGroupItem = Item;

        for (DWORD CarsNum = Automation->GroupCarsNum,
             CarIndex = 1; CarIndex <= CarsNum; CarIndex ++)
                {
                Automation->CarIndex = CarIndex;
                WideString Model = Automation->CarModel;

                AnsiString CarName;
                if (Model.IsEmpty())
                        CarName = FormatFloat("00000", Automation->CarDevice);
                else    {
                        CarName = Model;
                        WideString Number = Automation->CarNumber;
                        if (!Number.IsEmpty()) CarName += " - " + Number;
                        }

                TTreeNode *ChildItem = Items->AddChildObject(Item, CarName, (void *)CarIndex);
                if (ThisGroup && Automation->CarDevice == PrevCarDevice) SelCarItem = ChildItem;
                }
        }

if      (SelCarItem)   GroupsList->Selected = SelCarItem;
else if (SelGroupItem) GroupsList->Selected = SelGroupItem;
}

Items->AlphaSort(true);

Items->EndUpdate();
if (GroupsList->Selected) GroupsList->Selected->MakeVisible();
GroupsList->OnChange = GroupsListChange;
ShowPanels();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GroupsListCustomDrawItem(TCustomTreeView *Sender,
    TTreeNode *Node, TCustomDrawState State, bool &DefaultDraw)
{
if (Node) if (!Node->Level) GroupsList->Canvas->Font->Style = TFontStyles() << fsBold;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GroupsListChange(TObject *Sender, TTreeNode *Node)
{
ShowPanels();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowPanels()
{
TTreeNode *Node = GroupsList->Selected;
if (Node)
        {
        switch (Node->Level)
            {
            case 0:
                FillGroupInfoListView(Node);
                GroupInfoPanel->Visible = true;
                InfoPanel->Visible      = false;

                if (Automation)
                        {
                        PrevGroupFileName    = Automation->GroupFileName;
                        PrevCarDevice        = 0;
                        GroupsComboBox->Text = Automation->GroupFileName;
                        CarsComboBox->Text   = "";
                        }
                break;

            case 1:
                FillInfoListView(Node);
                FillOnlineInfoListView(Node);
                InfoPanel->Visible      = true;
                GroupInfoPanel->Visible = false;

                if (Automation)
                        {
                        PrevGroupFileName    = Automation->GroupFileName;
                        PrevCarDevice        = Automation->CarDevice;
                        GroupsComboBox->Text = Automation->GroupFileName;
                        CarsComboBox->Text   = FormatFloat("00000", Automation->CarDevice);
                        }
            }
        }
else    {
        GroupInfoPanel->Visible = false;
        InfoPanel->Visible      = false;

        PrevGroupFileName    = "";
        PrevCarDevice        = 0;
        GroupsComboBox->Text = "";
        CarsComboBox->Text   = "";
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FillGroupInfoListView(TTreeNode *Node)
{
TListItems *Items = GroupInfoListView->Items;
Items->BeginUpdate();

bool Clear = true;
if (Node && Automation)
        {
        Automation->GroupIndex = (int)Node->Data;

        Items->Item[0]->SubItems->Strings[0] = Automation->GroupName;
        Items->Item[1]->SubItems->Strings[0] = Automation->GroupAddress;
        Items->Item[2]->SubItems->Strings[0] = Automation->GroupContacts;
        Items->Item[3]->SubItems->Strings[0] = Automation->GroupFileName;
        Items->Item[4]->SubItems->Strings[0] = Automation->GroupCarsNum;

        Clear = false;
        }

if (Clear)
        for (int i = 0; i < Items->Count; i ++) Items->Item[i]->SubItems->Strings[0] = "";

Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FillInfoListView(TTreeNode *Node)
{
TListItems *Items = InfoListView->Items;
Items->BeginUpdate();

bool Clear = true;
if (Node && Automation)
        {
        Automation->GroupIndex = (int)Node->Parent->Data;
        Automation->CarIndex   = (int)Node->Data;

        Items->Item[0]->SubItems->Strings[0] = FormatFloat("00000", Automation->CarDevice);
        Items->Item[2]->SubItems->Strings[0] = Automation->CarModel;
        Items->Item[3]->SubItems->Strings[0] = Automation->CarNumber;
        Items->Item[4]->SubItems->Strings[0] = Automation->CarRoute;
        Items->Item[5]->SubItems->Strings[0] = Automation->CarDrivers;
        Items->Item[6]->SubItems->Strings[0] = Automation->CarAlias;
        Items->Item[7]->SubItems->Strings[0] = Automation->CarTelNum;
        Items->Item[9]->SubItems->Strings[0] = Automation->CarCheckPointsFile;

        Clear = false;
        }

if (Clear)
        {
        Items->Item[0]->SubItems->Strings[0] = "";
        for (int i = 2; i <= 7; i ++) Items->Item[i]->SubItems->Strings[0] = "";
        Items->Item[9]->SubItems->Strings[0] = "";
        }

Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FillOnlineInfoListView(TTreeNode *Node)
{             
TListItems *OnlineItems = OnlineListView->Items;
OnlineItems->BeginUpdate();

bool OnlineClear = true, CrdsClear = true, SensClear = true;
if (Node && Automation)
        {
        Automation->GroupIndex = (int)Node->Parent->Data;
        Automation->CarIndex   = (int)Node->Data;

        OnlineItems->Item[0]->SubItems->Strings[0] = "GPRS";

        try     {
                double CarLastTime = StrToDateTime(Automation->CarLastTime);
                OnlineItems->Item[1]->SubItems->Strings[0] =
                    FormatDateTime("dd.mm.yy - hh:mm:ss", CarLastTime);
                }
        catch (...)
                {
                OnlineItems->Item[1]->SubItems->Strings[0] = "";
                }

        try     {
                double CarLastATime = StrToDateTime(Automation->CarLastATime);
                OnlineItems->Item[2]->SubItems->Strings[0] =
                    FormatDateTime("dd.mm.yy - hh:mm:ss", CarLastATime);
                }
        catch (...)
                {
                OnlineItems->Item[2]->SubItems->Strings[0] = "";
                }

        OnlineClear = false;

        if (Automation->CarLatitude >= 0)
                {
                OnlineItems->Item[3]->SubItems->Strings[0] = GetCoordStr(Automation->CarLatitude)  + " ";
                OnlineItems->Item[4]->SubItems->Strings[0] = GetCoordStr(Automation->CarLongitude) + " ";

                double Angle = 450 - 180 * Automation->CarCourse / M_PI; if (Angle >= 360) Angle -= 360;
                AnsiString Str = FormatFloat("0.0", Angle);
                div_t hm = div(Angle * 2 + 0.5, 60);
                Str.cat_printf(" (%u:%02u)", hm.quot, hm.rem);
                OnlineItems->Item[5]->SubItems->Strings[0] = Str;

                double ParkTime = StrToTime(Automation->CarParkTime);
                if (ParkTime)
                        {
                        if (ParkTime > 1.0 / 24)
                                Str = " > 1 ";
                        else    Str = " " + ContinuanceStrMin(ParkTime);
                        }
                else    Str = " " + FormatFloat("#,##0.0 /", Automation->CarSpeed);
                OnlineItems->Item[6]->SubItems->Strings[0] = Str;

                CrdsClear = false;
                }

        DWORD Flags = Automation->CarFlags;
        if (Flags != 0xFFFFFFFF)
                {
                if (Flags & 0x02)
                        OnlineItems->Item[8]->SubItems->Strings[0] = SENS_ON;
                else    OnlineItems->Item[8]->SubItems->Strings[0] = SENS_OFF;

                if (Flags & 0x01)
                        OnlineItems->Item[9]->SubItems->Strings[0] = SENS_ON;
                else    OnlineItems->Item[9]->SubItems->Strings[0] = SENS_OFF;

                if (Automation->CarDevice < 21999)
                        for (int i = 10; i < 14; i ++)
                                OnlineItems->Item[i]->SubItems->Strings[0] = "-";
                else    for (int i = 10, mask = 0x10; i < 14; i ++, mask <<= 1)
                                if (Flags & mask)
                                        OnlineItems->Item[i]->SubItems->Strings[0] = SENS_ON;
                                else    OnlineItems->Item[i]->SubItems->Strings[0] = SENS_OFF;

                SensClear = false;
                }
        }

if (OnlineClear)
        for (int i = 0; i <= 2; i ++) OnlineItems->Item[i]->SubItems->Strings[0] = "";

if (CrdsClear)
        for (int i = 3; i <= 6; i ++) OnlineItems->Item[i]->SubItems->Strings[0] = "";

if (SensClear)
        for (int i = 8; i < OnlineItems->Count; i ++) OnlineItems->Item[i]->SubItems->Strings[0] = "";

OnlineItems->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowApplicationButtonClick(TObject *Sender)
{
Automation->ShowApplication();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HideApplicationButtonClick(TObject *Sender)
{
Automation->HideApplication();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LastPositionButtonClick(TObject *Sender)
{
Automation->GoToCarLastPosition(
    WideString(GroupsComboBox->Text),
    StrToInt(CarsComboBox->Text));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LastTrackButtonClick(TObject *Sender)
{
Automation->GoToCarLastTrack(
    WideString(GroupsComboBox->Text),
    StrToInt(CarsComboBox->Text));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadChPFromFileButtonClick(TObject *Sender)
{
LoadChPDialog->InitialDir = (AnsiString)Automation->WorkDirectory + "\\Points";
LoadChPDialog->FileName   = "";
if (LoadChPDialog->Execute())
    if (Automation)
        {
        WideString PointsName = ChangeFileExt(ExtractFileName(LoadChPDialog->FileName), "");
        Automation->LoadChPFromFile(PointsName);
        Automation->CheckPointIndex = 1;
        FillCheckPointsList();
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveChPToFileButtonClick(TObject *Sender)
{
SaveChPDialog->InitialDir = (AnsiString)Automation->WorkDirectory + "\\Points";
SaveChPDialog->FileName   = "";
if (SaveChPDialog->Execute())
    if (Automation)
        {
        WideString PointsName = ChangeFileExt(ExtractFileName(SaveChPDialog->FileName), "");
        Automation->SaveChPToFile(PointsName);
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearCheckPointsButtonClick(TObject *Sender)
{
Automation->ClearCheckPoints();
FillCheckPointsList();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddCheckPointButtonClick(TObject *Sender)
{
Application->CreateForm(__classid(TCheckPointsPropertiesForm), &CheckPointsPropertiesForm);
CheckPointsPropertiesForm->Caption = "  ";

if (CheckPointsPropertiesForm->ShowModal() == mrOk)
        {
        int LatGrad  = 0, LatMin  = 0, LatRem  = 0;
        int LongGrad = 0, LongMin = 0, LongRem = 0;
        int Radius = 30;

        sscanf(CheckPointsPropertiesForm->LatMaskEdit->Text.c_str(), "%u %u ,%u", &LatGrad,  &LatMin,  &LatRem);
        sscanf(CheckPointsPropertiesForm->LngMaskEdit->Text.c_str(), "%u %u ,%u", &LongGrad, &LongMin, &LongRem);
        sscanf(CheckPointsPropertiesForm->RadiusEdit->Text.c_str(),  "%u", &Radius);

        double Lat = LatGrad  + (1000 * LatMin  + LatRem)  / 60000.0;
        double Lng = LongGrad + (1000 * LongMin + LongRem) / 60000.0;
        if (Lat >  90) Lat =  90;
        if (Lng > 180) Lng = 180;
        if      (Radius <    10) Radius =    10;
        else if (Radius > 32500) Radius = 32500;

        if (Automation)
                {
                Automation->AddCheckPoint(
                    WideString(CheckPointsPropertiesForm->NameEdit->Text),
                    WideString(CheckPointsPropertiesForm->AddressEdit->Text),
                    WideString(CheckPointsPropertiesForm->IDEdit->Text),
                    Lat, Lng, Radius);
                FillCheckPointsList();
                }
        }

delete CheckPointsPropertiesForm;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ChangeCheckPointButtonClick(TObject *Sender)
{
Application->CreateForm(__classid(TCheckPointsPropertiesForm), &CheckPointsPropertiesForm);
CheckPointsPropertiesForm->Caption = "  ";

CheckPointsPropertiesForm->NameEdit->Text    = Automation->CheckPointName;
CheckPointsPropertiesForm->AddressEdit->Text = Automation->CheckPointAddress;
CheckPointsPropertiesForm->IDEdit->Text      = Automation->CheckPointID;
CheckPointsPropertiesForm->RadiusEdit->Text  = Automation->CheckPointRadius;

AnsiString Str;
div_t L, R;
L = div(Automation->CheckPointLatitude  * 100000 + 0.5, 100000), R = div(0.6 * L.rem + 0.5, 1000);
Str.printf("%u %02u,%03u", L.quot, R.quot, R.rem);
CheckPointsPropertiesForm->LatMaskEdit->Text = Str;
L = div(Automation->CheckPointLongitude * 100000 + 0.5, 100000), R = div(0.6 * L.rem + 0.5, 1000);
Str.printf("%u %02u,%03u", L.quot, R.quot, R.rem);
CheckPointsPropertiesForm->LngMaskEdit->Text = Str;

if (CheckPointsPropertiesForm->ShowModal() == mrOk)
        {
        int LatGrad  = 0, LatMin  = 0, LatRem  = 0;
        int LongGrad = 0, LongMin = 0, LongRem = 0;
        int Radius = 30;

        sscanf(CheckPointsPropertiesForm->LatMaskEdit->Text.c_str(), "%u %u ,%u", &LatGrad,  &LatMin,  &LatRem);
        sscanf(CheckPointsPropertiesForm->LngMaskEdit->Text.c_str(), "%u %u ,%u", &LongGrad, &LongMin, &LongRem);
        sscanf(CheckPointsPropertiesForm->RadiusEdit->Text.c_str(),  "%u", &Radius);

        double Lat = LatGrad  + (1000 * LatMin  + LatRem)  / 60000.0;
        double Lng = LongGrad + (1000 * LongMin + LongRem) / 60000.0;
        if (Lat >  90) Lat =  90;
        if (Lng > 180) Lng = 180;
        if      (Radius <    10) Radius =    10;
        else if (Radius > 32500) Radius = 32500;

        if (Automation)
                {
                Automation->CheckPointName      = WideString(CheckPointsPropertiesForm->NameEdit->Text);
                Automation->CheckPointAddress   = WideString(CheckPointsPropertiesForm->AddressEdit->Text);
                Automation->CheckPointID        = WideString(CheckPointsPropertiesForm->IDEdit->Text);
                Automation->CheckPointLatitude  = Lat;
                Automation->CheckPointLongitude = Lng;
                Automation->CheckPointRadius    = Radius;
                FillCheckPointsList();
                }
        }

delete CheckPointsPropertiesForm;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DeleteCheckPointButtonClick(TObject *Sender)
{
Automation->DeleteCheckPoint();
FillCheckPointsList();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FillCheckPointsList()
{
TListItems *Items = ChPListView->Items;
Items->BeginUpdate();
Items->Clear();

if (Automation)
        {
        DWORD CheckPointsNum = Automation->CheckPointsNum;
        DWORD CurrIndex      = Automation->CheckPointIndex;
        if (CurrIndex > CheckPointsNum) CurrIndex = CheckPointsNum;

        for (DWORD CheckPointIndex = 1; CheckPointIndex <= CheckPointsNum; CheckPointIndex ++)
                {
                Automation->CheckPointIndex = CheckPointIndex;

                TListItem *Item = Items->Add();

                Item->Caption = Automation->CheckPointName;
                Item->Data    = (void *)CheckPointIndex;

                TStrings *SubItems = Item->SubItems;
                SubItems->Capacity = 5;
                SubItems->Append(Automation->CheckPointAddress);
                SubItems->Append(Automation->CheckPointID);
                SubItems->Append(GetCoordStr(Automation->CheckPointLatitude)  + " ");
                SubItems->Append(GetCoordStr(Automation->CheckPointLongitude) + " ");
                SubItems->Append(Automation->CheckPointRadius);
                }

        if (CurrIndex)
                {
                Automation->CheckPointIndex = CurrIndex;
                TListItem *Item = Items->Item[CurrIndex - 1];
                Item->Focused  = true;
                Item->Selected = true;
                Item->MakeVisible(false);
                }
                
        ChangeCheckPointButton->Enabled = CurrIndex;
        DeleteCheckPointButton->Enabled = CurrIndex;
        }

Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ChPListViewSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
if (Selected && Item && Automation) Automation->CheckPointIndex = (int)Item->Data;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GroupsComboBoxDropDown(TObject *Sender)
{
TStrings *Items = GroupsComboBox->Items;
Items->BeginUpdate();
Items->Clear();

if (Automation)
for (DWORD GroupsNum = Automation->GroupsNum,
     GroupIndex = 1; GroupIndex <= GroupsNum; GroupIndex ++)
        {
        Automation->GroupIndex = GroupIndex;
        Items->AddObject(Automation->GroupFileName, (TObject *)GroupIndex);
        }

Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CarsComboBoxDropDown(TObject *Sender)
{
TStrings *Items = CarsComboBox->Items;
Items->BeginUpdate();
Items->Clear();

if (Automation)
        {
        Automation->SetGroupIndexByFileName(WideString(GroupsComboBox->Text));
        for (DWORD CarsNum = Automation->GroupCarsNum,
             CarIndex = 1; CarIndex <= CarsNum; CarIndex ++)
                {
                Automation->CarIndex = CarIndex;
                Items->AddObject(FormatFloat("00000", Automation->CarDevice),
                    (TObject *)CarIndex);
                }
        }

Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CheckPointsBoxDropDown(TObject *Sender)
{
TStrings *Items = CheckPointsBox->Items;
Items->BeginUpdate();
Items->Clear();

if (Automation)
        {
        TSearchRec SearchData;
        if (!FindFirst((AnsiString)Automation->WorkDirectory +
            "\\Points\\*.chp", faAnyFile, SearchData))
                {
                do Items->Append(ChangeFileExt(SearchData.Name, ""));
                while (!FindNext(SearchData));
                FindClose(SearchData);
                }
        }

Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetChPPrmsButtonClick(TObject *Sender)
{
Automation->SetCarChPParameters(
    WideString(GroupsComboBox->Text),
    StrToInt(CarsComboBox->Text),
    DefaultChPPrmsBox->Checked,
    WideString(CheckPointsBox->Text),
    CallPointsByParkBox->Checked,
    StrToInt(CallPointsByParkTimeEdit->Text),
    CallPointsBySensBox->Checked,
    CallPointsBySensIndexBox->ItemIndex + 1,
    StrToInt(CallPointsBySensTimeEdit->Text));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ComputingButtonClick(TObject *Sender)
{
TDateTime FirstTime = FirstDatePicker->Date;
TDateTime LastTime  = LastDatePicker->Date;
ReplaceTime(FirstTime, FirstTimePicker->Time);
ReplaceTime(LastTime,  LastTimePicker->Time);
//Automation->ComputingTimeout = 3;
//Automation->WaitForComputing(
Automation->StartComputing(
    WideString(GroupsComboBox->Text),
    StrToInt(CarsComboBox->Text),
    WideString(FormatDateTime("dd.mm.yyyy hh:mm:ss", FirstTime)),
    WideString(FormatDateTime("dd.mm.yyyy hh:mm:ss", LastTime)),
    WideString(GSMSourceButton->Checked ? "GSM" : "USB"),
    TakeTripsSetupBox->Checked);
ComputingTimerTimer(ComputingTimer);
ComputingTimer->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ComputingTimerTimer(TObject *Sender)
{
if (Automation)
        {
        if (Automation->ComputingBusy)
                {
                ComputingTimer->Enabled = true;
                ComputingStatusInfoLabel->Caption = "";
                ComputingProgressBar->Position = Automation->ComputingPercents;
                }
        else    {
                ComputingTimer->Enabled = false;
                ComputingStatusInfoLabel->Caption = "";
                ComputingProgressBar->Position = 0;
                }
        }
else    {
        ComputingTimer->Enabled = false;
        ComputingStatusInfoLabel->Caption = "";
        ComputingProgressBar->Position = 0;
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ComputingResultButtonClick(TObject *Sender)
{
PageControl->ActivePage = TripsInfoSheet;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FillTripsInfoList()
{
TListItems *Items = TripsListView->Items;
Items->BeginUpdate();
Items->Clear();

if (Automation)
        {
        DWORD TripsNum  = Automation->TripsNum;
        DWORD CurrIndex = Automation->TripIndex;
        if (!CurrIndex) CurrIndex = 1;
        if (CurrIndex > TripsNum) CurrIndex = TripsNum;

        for (DWORD TripIndex = 1; TripIndex <= TripsNum; TripIndex ++)
                {
                Automation->TripIndex = TripIndex;

                TListItem *Item = Items->Add();

                Item->Caption = TripIndex;
                Item->Data    = (void *)TripIndex;

                TStrings *SubItems = Item->SubItems;
                SubItems->Capacity = 15;
                SubItems->Append(Automation->TripStartRealTime);
                SubItems->Append(Automation->TripEndRealTime);
                SubItems->Append(Automation->TripRealCont);
                SubItems->Append(Automation->TripMoveCont);
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripMaxSpeed));
                if (Automation->TripAverSpeed >= 0)
                        SubItems->Append(FormatFloat("#,##0.0", Automation->TripAverSpeed));
                else    SubItems->Append("-");
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripDistance / 1000));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripTank1StartLevel));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripTank1EndLevel));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripTank2StartLevel));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripTank2EndLevel));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripMotor1Hours));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripMotor2Hours));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripMotor1Fuel));
                SubItems->Append(FormatFloat("#,##0.0", Automation->TripMotor2Fuel));
                }

        if (CurrIndex)
                {
                Automation->TripIndex = CurrIndex;
                TListItem *Item = Items->Item[CurrIndex - 1];
                Item->Focused  = true;
                Item->Selected = true;
                Item->MakeVisible(false);
                }
        else    ExportToFileButton->Enabled = false;

        EntriesTypeBox->Enabled = CurrIndex;
        EntriesButton->Enabled  = CurrIndex;
        }

Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TripsListViewSelectItem(TObject *Sender, TListItem *Item, bool Selected)
{
if (Selected && Item && Automation)
        {
        Automation->TripIndex = (int)Item->Data;
        ExportToFileButton->Enabled = Automation->TripCrdEntriesNum;
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EntriesButtonClick(TObject *Sender)
{
switch (EntriesTypeBox->ItemIndex)
    {
    case  0: Automation->TripEntriesListTypeName = WideString("checkpoints");
             Automation->TripEntriesListKindName = WideString("points");     break; // . 

    case  1: Automation->TripEntriesListTypeName = WideString("checkpoints");
             Automation->TripEntriesListKindName = WideString("stages");     break; // . 

    case  2: Automation->TripEntriesListTypeName = WideString("parks");      break; // 

    case  3: Automation->TripEntriesListTypeName = WideString("blinds");     break; // 

    case  4: Automation->TripEntriesListTypeName = WideString("power_off");  break; // 

    case  5: Automation->TripEntriesListTypeName = WideString("over_speed"); break; // 

    case  6: Automation->TripEntriesListTypeName = WideString("tank");
             Automation->TripEntriesListKindName = WideString("num1");       break; //  1

    case  7: Automation->TripEntriesListTypeName = WideString("tank");
             Automation->TripEntriesListKindName = WideString("num2");       break; //  2

    case  8: Automation->TripEntriesListTypeName = WideString("sensor_on");
             Automation->TripEntriesListKindName = WideString("num1");       break; //  1

    case  9: Automation->TripEntriesListTypeName = WideString("sensor_on");
             Automation->TripEntriesListKindName = WideString("num2");       break; //  2

    case 10: Automation->TripEntriesListTypeName = WideString("sensor_on");
             Automation->TripEntriesListKindName = WideString("num3");       break; //  3

    case 11: Automation->TripEntriesListTypeName = WideString("sensor_on");
             Automation->TripEntriesListKindName = WideString("num4");       break; //  4

    case 12: Automation->TripEntriesListTypeName = WideString("sensor_on");
             Automation->TripEntriesListKindName = WideString("num5");       break; //  5

    case 13: Automation->TripEntriesListTypeName = WideString("sensor_on");
             Automation->TripEntriesListKindName = WideString("num6");       break; //  6
    }

Application->CreateForm(__classid(TPointsListForm), &PointsListForm);

TListItems *Items = PointsListForm->PointsListView->Items;
Items->BeginUpdate();
Items->Clear();

DWORD EntriesNum = Automation->TripEntriesNum;
for (DWORD EntryIndex = 1; EntryIndex <= EntriesNum; EntryIndex ++)
        {
        Automation->EntryIndex = EntryIndex;

        TListItem *Item = Items->Add();
        
        Item->Caption = Automation->EntryID;
        Item->Data    = (void *)EntryIndex;

        TStrings *SubItems = Item->SubItems;
        SubItems->Capacity = 14;
        SubItems->Append(Automation->EntryStartName);
        SubItems->Append(Automation->EntryStartRealTime);
        SubItems->Append(Automation->EntryEndName);
        SubItems->Append(Automation->EntryEndRealTime);
        SubItems->Append(Automation->EntryRealCont);
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryDistance / 1000));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryTank1StartLevel));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryTank1EndLevel));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryTank2StartLevel));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryTank2EndLevel));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryMotor1Hours));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryMotor2Hours));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryMotor1Fuel));
        SubItems->Append(FormatFloat("#,##0.0", Automation->EntryMotor2Fuel));
        }

Items->EndUpdate();

PointsListForm->ShowModal();
delete PointsListForm;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExportToFileButtonClick(TObject *Sender)
{
if (ExportTrackDialog->Execute())
    if (Automation)
        {
        const AnsiString FileTypes[] = {".plt", ".kml"};
        ExportTrackDialog->FileName =
            ChangeFileExt(ExportTrackDialog->FileName.Trim(),
            FileTypes[ExportTrackDialog->FilterIndex - 1]);
        Automation->ExportTrackToFile(WideString(ExportTrackDialog->FileName),
            1, WideString(""), WideString(""), 0, 0, 0);
        }
}
//---------------------------------------------------------------------------

